/**************************************************************

   Program Title:  SmartLCD Controller
   
   Program Name:  SmartLCD_Init.c		  Version: 0.88
   
   Author:  SS	  					  Date:  August 10, 2008
   
   Compiled with ImageCraft ICCAVR Standard V7.13A
   
   This file contains routines that implement the basic initialization 
   routines for the SmartLCD Controller.
   
   Copyright 2004-2008, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2008, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: SmartLCD_Init.c,v $
 * Revision 0.86  2006/02/07 16:40:20  SS
 *    2-wire interface, 8MHz clock, AVR Flash writer, XON/XOFF, 38,700baud
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 *
 */
#include <iom168v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
//#include <stdio.h>
#include "SmartLCD_Uart.h"
#include "SmartLCD_Defines.h"
#include "SmartLCD_Extern.h"

//****************************************************************
//****************************************************************
// Initialization code
//****************************************************************
//****************************************************************

// Initialize UART
void InitUART(unsigned char baudrate)
{
	unsigned char x;
	
	// Set the baud rate
	UBRR0H = 0x00;
	UBRR0L = baudrate; 
	// Enable UART receiver and transmitter
	UCSR0B = BIT(RXCIE0)|BIT(UDRIE0)|BIT(RXEN0)|BIT(TXEN0); 
    //  8 data bits, 2 stop bits
//	UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);
    //  8 data bits, 1 stop bit
//	UCSR0C = (1<<UMSEL01)|(3<<UCSZ01);
	UCSR0C = (3<<UCSZ00);

	x = 0; // flush receive buffer
	rxcnt = x; // rx buffer empty
	xoffsent = NO;
	sendxoff = NO;
	xonsent = NO;
	checkflag = NO;
	enablexonoff = YES; // turn serial driver xon/xoff flow control on
	echoinchars = NO;   // echo all received characters
	UART_RxTail = x;
	UART_RxHead = x;
	UART_TxTail = x;
	UART_TxHead = x;
	dataoverrun = NO;
	frameerror = NO;
	xoncnt = 0;
	xoffcnt = 0;
}

//  Setup the Analog to Digital converter
//  TBD - convert to background interrupt process
//	  AD0 - ADC0 - battery voltage - use internal voltage ref - 1.1v
//    AD1 - ADC6
//    AD2 - ADC7
void Init_ADC(void)
{
	DDRC &= ~0x01;  // Make ADC0 - PC0 an input
		   		    // this version does not use int service
					// so no setup required here
}

//  Start the real time clock 
//  Clocked by external 32.768KHz watch crystal
//  Set prescalar to 0, set to interrupt using compare match 
//  set to 128 (4msec interrupt rate).  
void InitRealTimeClock(void)
{  
    // Software counters - approximately 4msec resolution
	rtccountdown1 = 0;  
	rtccountdown2 = 0;
	rtccountdown3 = 0;
	rtccountdown4 = 0;
	
	seconds = minutes = hours = days = 0;
	rtcclockcnt = 0;
	
 	TCNT2 = 0;
 	// Select external real time clock oscillator
	ASSR = (1<<AS2);
	// Set match interrupt to 4msec
	OCR2A = 127;
	// Reload counter on match interrupt - pre-scalar set to 0
	//  CTC mode
	TCCR2A = (1<<WGM21);   
    // Setup interrupts for real time clock
	TCCR2B = (1<<CS20); 
    TIMSK2 |= (1<<OCIE2A);
}

// Timer2 is called every 1msec - used to maintain four count down 
// software timers to control precise time delays in various routines
void Init_Timer0(void)
{
 	 mseccountdown1 = 0;
	 mseccountdown2 = 0;
	 mseccountdown3 = 0;
	 mseccountdown4 = 0;
	 
	 TCNT0 = 0;
	 // Set mode 2 - reset counter on compare match, clk/8
	 TCCR0B = ((1<<CS01)|(1<<CS00));
	 TCCR0A = (1<<WGM01);
	 // Reset count after 1 msec
	 OCR0A = 125;	 // Enable compare match interrupt vector
	 TIMSK0 |= (1<<OCIE0A);
}

// Setup watch dog timer - TBD
void InitWatchdog(void)
{
}

void Init_SPI_Master(void)
{
    // Set MOSI and SCK output, all others input
	DDRB = (1<<PB5)|(1<<PB7)|(1<<PB4);
	PORTB |= 0xA0;  // Set bits MOSI & SCK

	//  Make Flash Chip select PC4 output and set high
	DDRC |= 0x10; 
	PORTC |= 0x10;  
	
	// Enable SPI, Master, set clock rate fck
	//    Select SPI Mode 3 - CPOL=1, CPHA=1
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<CPOL)|(1<<CPHA)|(1<<SPR0);
//	SPCR = 0x91;
}

//************************************************************
//  Initialize the AVR - Set LED start conditions
//************************************************************
void Init_AVR(void)
{
 	 // Setup Port B
     DDRB = 0x07;	     // LCD Control Port settings - outputs
	 	  				 //   PB0 - RES   LCD Reset
						 //   PB1 - CS    LCD CS
						 //   PB2 - A0    LCD command/data 
	 PORTB |= 0x07;
						 
//	 LCDDATA = 0xFF;	//all off
	
	 // Setup Port C
	 DDRC |= 0x0C;       // LCD Serial Data Port settings - outputs
//	 LCDCONTROL = 0x01;  // PC2 - SI    - LCD serial data
					     // PC3 - SCLK  - LCD serial clock

	// Setup Port D
	DDRD = 0x60;   //   PD2 - Switch 1
		   		   //   PD3 - Switch 2
				   //   PD4 - Switch 3
		   		   //   PD5 - LED
		   		   //   PD6 - LCD LED
				   //   PD7 - Switch 4
	PORTD = 0x20;  // Turn all LED off and LCD LED off
	
	//   Setup SPI - SPI default inactive in this release
//    Init_SPI_Master();
	
	//  Initialize ADC
	Init_ADC();
	
	//  Turn on real time clock on PB6,7 Timer/Counter 2
	//  Requires 32.768KHz crystal installed
	InitRealTimeClock();
	
	// Setup Timer 0 to interrupt a 1msec rate - this interrupt
	// controls four software delay timers used by various 
	// routines
	Init_Timer0();
	
	//    Setup 2-Wire Interface
	//Init_TWI_Master();   // SCL freq = sclk/(16+2*TWBR)*4*TWPS

	// Watchdog timer setup
//	InitWatchdog();
	
	// Initialize UART - Interrupt
	// Set the baudrate to 9,600 bps using an 8MHz clock
	InitUART(BAUDRATE19200); 
//	InitUART(BAUDRATE9600); 

	textdisplaymode = INACTIVE;
	lcdledpower = OFF;
					
	_SEI(); // Enable interrupts => enable UART interrupts
}
